@@ -49,9 +49,13 @@
 #include <ctype.h>
 #include <limits.h>
 
+#ifndef BYTE_ORDER
+#error "no byte order defined"
+#endif
+
 struct a_sound {
 	u_int len;
-	int8_t *buf;
+	void *buf;
 };
 
 struct play_list {
@@ -75,6 +79,8 @@ struct s_params {
 	u_int sp_inCharlen;	/* interCharacter length */
 	u_int sp_sampthresh;	/* threshold for queuing more audio */
 	u_int sp_blocksize;	/* audio block size */
+	u_int sp_channels;	/* number of channels */
+	u_int sp_precision;	/* sample precision (bits) */
 	int sp_fd;		/* audio file descriptor */
 	int sp_seenspace;	/* seen a space character? */
 };
@@ -125,7 +131,7 @@ xx_malloc(size_t sz, const char *fname, const int lineno)
 
 	m = malloc(sz);
 	if (m == NULL)
-		err(1, "malloc");
+		err(1, "malloc(%s:%d:%ld", fname, lineno, (long)sz);
 	if (diagmode > 2)
 		printf("M:%p:%lu:%s:%d\n", m, (unsigned long)sz,
 		    fname, lineno);
@@ -428,6 +434,7 @@ int
 build_inChar(struct a_sound *snd, struct s_params *pars)
 {
 	float samplen, charu;
+	u_int dit_samps;
 
 	if (overallwpm >= charwpm) {
 		float u;
@@ -442,8 +449,11 @@ build_inChar(struct a_sound *snd, struct s_params *pars)
 		Tc = (3.0 * Ta) / 19.0;
 		samplen = Tc * (float)pars->sp_rate;
 	}
-	charu = ((float)pars->sp_ditlen) / 2.0;
-	snd->len = rintf(samplen - charu);
+	dit_samps = pars->sp_ditlen /
+	    (pars->sp_channels * (pars->sp_precision / 8));
+	charu = ((float)dit_samps) / 2.0;
+	snd->len = rintf(samplen - charu) *
+	    pars->sp_channels * (pars->sp_precision / 8);
 	pars->sp_inCharlen = snd->len;
 	return (build_silence(snd, pars));
 }
@@ -452,6 +462,7 @@ int
 build_inWord(struct a_sound *snd, struct s_params *pars)
 {
 	float samplen, m;
+	float inCharSamp, ditsamp;
 
 	if (overallwpm >= charwpm) {
 		float u;
@@ -466,8 +477,13 @@ build_inWord(struct a_sound *snd, struct s_params *pars)
 		Tc = (7.0 * Ta) / 19.0;
 		samplen = Tc * (float)pars->sp_rate;
 	}
-	m = (float)pars->sp_inCharlen + ((float)pars->sp_ditlen / 2.0);
-	snd->len = (u_int)rintf(samplen - m);
+	ditsamp = ((float)pars->sp_ditlen / 2.0) /
+	    (pars->sp_channels * (pars->sp_precision / 8));
+	inCharSamp = pars->sp_inCharlen /
+	    (pars->sp_channels * (pars->sp_precision / 8));
+	m = inCharSamp + ditsamp;
+	snd->len = (u_int)rintf(samplen - m) *
+	    pars->sp_channels * (pars->sp_precision / 8);
 	return (build_silence(snd, pars));
 }
 
@@ -505,11 +521,23 @@ build_dah(struct a_sound *snd, struct s_params *pars)
 int
 build_snd(struct a_sound *snd, struct s_params *pars, double units)
 {
-	u_int i, attack1, attack2, attack3;
-	float u, m;
+	u_int i, j, attack1, attack2, attack3, nsamps, idx;
+	float u, m, prec;
+
+	switch (pars->sp_precision) {
+	case 8:
+		prec = 127.0;
+		break;
+	case 16:
+		prec = 32767.0;
+		break;
+	default:
+		return (-1);
+	}
 
 	u = 1.2 / charwpm;
-	snd->len = rintf((units + 1.0) * u * (float)pars->sp_rate);
+	nsamps = rintf((units + 1.0) * u * (float)pars->sp_rate);
+	snd->len = nsamps * pars->sp_channels * (pars->sp_precision / 8);
 	snd->buf = (int8_t *)x_malloc(snd->len);
 	if (snd->buf == NULL)
 		return (-1);
@@ -524,7 +552,8 @@ build_snd(struct a_sound *snd, struct s_params *pars, double units)
 	attack1 = (u_int)u;
 	attack3 = attack1 + attack2;
 
-	for (i = 0; i < snd->len; i++) {
+
+	for (i = 0, idx = 0; i < nsamps; i++) {
 		double d;
 		float t = (double)i / (double)pars->sp_rate;
 
@@ -545,8 +574,20 @@ build_snd(struct a_sound *snd, struct s_params *pars, double units)
 		else
 			m = 1.0;
 
-		d = rintf(m * 127.0 * sinf(t * 2 * M_PI * pars->sp_hz));
-		snd->buf[i] = d;
+		d = rintf(m * prec * sinf(t * 2 * M_PI * pars->sp_hz));
+
+		if (pars->sp_precision == 8) {
+			for (j = 0; j < pars->sp_channels; j++) {
+				int8_t *bp = (int8_t *)snd->buf;
+
+				bp[idx++] = d;
+			}
+		} else if (pars->sp_precision == 16) {
+			for (j = 0; j < pars->sp_channels; j++) {
+				int16_t *bp = (int16_t *)snd->buf;
+				bp[idx++] = d;
+			}
+		}
 	}
 	return (0);
 }
@@ -625,9 +666,11 @@ time_check(struct s_params *pars)
 	float sampmin;
 	float perword, e, m;
 
-	perword = (10.0 * (float)dit.len) + (4.0 * (float)dah.len) +
-	    (5.0 * (float)inChar.len) + (1.0 * (float)inWord.len);
-
+	perword = 0;
+	perword += 10 * dit.len / pars->sp_channels / (pars->sp_precision / 8);
+	perword += 4 * dah.len / pars->sp_channels / (pars->sp_precision / 8);
+	perword += 5 * inChar.len / pars->sp_channels / (pars->sp_precision / 8);
+	perword += 1 * inWord.len / pars->sp_channels / (pars->sp_precision / 8);
 	sampmin = (float)pars->sp_rate * 60.0;
 	m = sampmin/perword;
 	e = (fabsf(m - overallwpm)/overallwpm) * 100;
@@ -763,12 +806,24 @@ main(int argc, char *argv[])
 		err(1, "open %s", afname);
 
 	AUDIO_INITINFO(&ai);
-	ai.play.sample_rate = 22050;
+	/* ai.play.sample_rate = 22050; */
 	ai.play.encoding = AUDIO_ENCODING_SLINEAR;
 	ai.play.precision = 8;
 	ai.play.channels = 1;
-	if (ioctl(f, AUDIO_SETINFO, &ai) == -1)
-		err(1, "setinfo");
+	if (ioctl(f, AUDIO_SETINFO, &ai) == -1) {
+#if BYTE_ORDER == LITTLE_ENDIAN
+		ai.play.encoding = AUDIO_ENCODING_SLINEAR_LE;
+#elif BYTE_ORDER == BIG_ENDIAN
+		ai.play.encoding = AUDIO_ENCODING_SLINEAR_BE;
+#else
+# error "oh please, get the pdp outta here."
+#endif
+		ai.play.precision = 16;
+		ai.play.channels = 2;
+		if (ioctl(f, AUDIO_SETINFO, &ai) == -1) {
+			err(1, "setinfo");
+		}
+	}
 
 	if (ioctl(f, AUDIO_GETINFO, &ai) == -1)
 		err(1, "getinfo");
@@ -778,6 +833,8 @@ main(int argc, char *argv[])
 	pars.sp_blocksize = ai.blocksize;
 	pars.sp_seenspace = 0;
 	pars.sp_fd = f;
+	pars.sp_channels = ai.play.channels;
+	pars.sp_precision = ai.play.precision;
 
 	if (diagmode > 0) {
 		check_chars();
